@@ -1,10 +1,10 @@
 """ High-level world utility and convenience functions. """
-
+from phi.geom import union
 from .world import StateProxy, World
-from .field.mask import union_mask
+from .field.mask import mask
 
 
-def obstacle_mask(world_or_proxy):
+def obstacle_mask(world_or_proxy, **mask_kwargs):
     """
 Builds a binary Field, masking all obstacles in the world.
     :param world_or_proxy: World or StateProxy object
@@ -13,4 +13,4 @@ Builds a binary Field, masking all obstacles in the world.
     world = world_or_proxy.world if isinstance(world_or_proxy, StateProxy) else world_or_proxy
     assert isinstance(world, World)
     geometries = [obstacle.geometry for obstacle in world.state.all_with_tag('obstacle')]
-    return union_mask(geometries)
+    return mask(union(geometries), **mask_kwargs)
