@@ -62,7 +62,7 @@ See the struct documentation at documentation/Structs.ipynb
         self._set_items(**kwargs)
         for trait in self.__traits__:
             trait.endow(self)
-        if content_type is VALID:
+        if content_type is not INVALID:
             self.validate()
 
     @derived()
@@ -154,13 +154,17 @@ Performs validation on this struct if it holds data and is invalid.
 Data-holding structs should always be valid while structs holding non-data content such as shapes or data types are not regarded as valid.
         :return: True if validation was performed, False otherwise
         """
-        if not skip_validate() and self.__content_type__ is INVALID:
+        if not skip_validate() and self.__can_validate__():
             self.__validate__()
-            self.__content_type__ = VALID
+            if self.__content_type__ is INVALID:
+                self.__content_type__ = VALID
             return True
         else:
             return False
 
+    def __can_validate__(self):
+        return self.__content_type__ is INVALID
+
     def __validate__(self):
         for trait in self.__traits__:
             trait.pre_validate_struct(self)
@@ -323,4 +327,3 @@ def equal(obj1, obj2):
             if obj1 is not obj2:
                 return False
     return True
-
